module SpectralIndices

using Dates
using Downloads
using JSON

abstract type AbstractSpectralIndex end
abstract type AbstractPlatformBand end

include("utils.jl")
include("axioms.jl")
include("compute_index.jl")
include("compute_kernel.jl")
#include("datasets.jl")

if !isdefined(Base, :get_extension)
    include("../ext/SpectralIndicesDataFramesExt.jl")
    include("../ext/SpectralIndicesYAXArraysExt.jl")
end

indices = _create_indices()
bands = _create_bands()
constants = _create_constants()

export SpectralIndex, indices, compute
export PlatformBand, Band
export Constant
export compute_index
export compute_kernel, linear, poly, RBF

for (name, instance) in indices
    @eval begin
        $(Symbol(name)) = $instance
        export $(Symbol(name))
    end
end

end #module
